package com.arms.api.analysis.requirement.controller;

import com.arms.api.analysis.requirement.dto.ReqAnalysisAggrDTO;
import com.arms.api.analysis.requirement.vo.ReqAndIssueTotalCountVO;
import com.arms.api.issue.almapi.model.entity.AlmIssueEntity;
import com.arms.api.util.model.dto.request.지라이슈_기본_검색_집계_하위_요청;
import com.arms.api.util.model.dto.response.*;
import com.arms.api.util.model.dto.request.ALM_이슈_제품_및_버전_집계_요청;
import com.arms.api.util.model.dto.request.ALM_이슈_일별_제품_및_버전_집계_요청;
import com.arms.egovframework.javaservice.esframework.model.vo.버킷_집계_결과;
import com.arms.egovframework.javaservice.esframework.model.vo.버킷_집계_결과_목록_합계;

import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Map;

import com.arms.api.analysis.requirement.service.ReqAnalysisService;

@RestController
@RequestMapping("/engine")
@Slf4j
@AllArgsConstructor
public class ReqAnalysisController {
    //
    // 요구관리, 현황관리, 간트, 칸반에서 사용하는 API 를 넣는다(?)
    //
    private final ReqAnalysisService reqAnalysisService;


    // 비용분석
    @PostMapping("/jira/dashboard/aggregation/flat")
    public ResponseEntity<버킷_집계_결과_목록_합계> flatAggregation(
            @RequestBody ALM_이슈_제품_및_버전_집계_요청 집계_요청
    ) {

        log.info("[요구사항_분석_컨트롤러 :: flatAggregation]");

        return ResponseEntity.ok(reqAnalysisService.flatAggregation(집계_요청));
    }

    // 대시보드, 디테일_대시보드
    @PostMapping("/jira/dashboard/requirements-jira-issue-statuses")
    public ResponseEntity<Map<String, 요구사항_지라이슈상태_주별_집계>> 요구사항이슈월별집계(
            @RequestBody ALM_이슈_제품_및_버전_집계_요청 지라이슈_제품_및_제품버전_집계_요청
    ) {
        log.info("[요구사항_분석_컨트롤러 :: 요구사항이슈월별집계]");

        return ResponseEntity.ok(reqAnalysisService.요구사항_지라이슈상태_주별_집계(지라이슈_제품_및_제품버전_집계_요청));
    }

    // Dashboard, Detail_Dashboard, Analysis Resource
    @PostMapping("/jira/dashboard/version-assignees")
    public ResponseEntity<List<버킷_집계_결과>> 제품별_버전_및_작업자(
            @RequestBody ALM_이슈_제품_및_버전_집계_요청 지라이슈_제품_및_제품버전_집계_요청
    ) {
        log.info("[요구사항_분석_컨트롤러 :: 제품별_버전_및_작업자] :: 담당자_목록");

        return ResponseEntity.ok(reqAnalysisService.제품_버전별_담당자_목록(지라이슈_제품_및_제품버전_집계_요청));
    }

    // Top-menu
    @PostMapping("/analysis/top-menu/totalCountWithReqAndSubtask")
    public ResponseEntity<ReqAndIssueTotalCountVO> topMenuTotalCountWithReqAndSubtask(@RequestBody ReqAnalysisAggrDTO reqAnalysisAggrDTO) {

        log.info("[요구사항_분석_컨트롤러 :: 제품별_버전_및_작업자] :: 담당자_목록");

        return ResponseEntity.ok(reqAnalysisService.topMenuTotalCountWithReqAndSubtask(reqAnalysisAggrDTO));
    }

    // Top-menu, getProgess - 대체 예정
    @PostMapping("/jira/dashboard/normal-version-only/{pdServiceId}")
    public ResponseEntity<버킷_집계_결과_목록_합계> 일반_버전필터_집계(@PathVariable Long pdServiceId,
        @RequestBody 지라이슈_기본_검색_집계_하위_요청 지라이슈_일반_집계_요청) {

        log.info("[요구사항_분석_컨트롤러 :: 제품별_버전_및_작업자] :: 담당자_목록");

        return ResponseEntity.ok(reqAnalysisService.normalVersionOnly(pdServiceId,지라이슈_일반_집계_요청));
    }

    // Gantt - reqProgress
    @PostMapping("/jira/dashboard/normal-version/resolution")
    public ResponseEntity<버킷_집계_결과_목록_합계> 일반_버전필터_해결책유무_검색(
                                    @RequestBody ALM_이슈_제품_및_버전_집계_요청 지라이슈_제품_및_제품버전_집계_요청,
                                    @RequestParam(required = false) String resolution) {

        log.info("[요구사항_분석_컨트롤러 :: 일반_버전필터_해결책유무_검색]");

        return ResponseEntity.ok(reqAnalysisService.일반_버전필터_해결책유무_검색(지라이슈_제품_및_제품버전_집계_요청, resolution));
    }

    // Analysis Cost
    @PostMapping("/jira/dashboard/version-req-assignees")
    public ResponseEntity<List<버킷_집계_결과>> 제품별_버전_및_요구사항별_작업자(
            @RequestBody ALM_이슈_제품_및_버전_집계_요청 지라이슈_제품_및_제품버전_집계_요청
    ) {
        log.info("요구사항 분석 컨트롤러 :: 제품별_버전_및_요구사항별_작업자");

        return ResponseEntity.ok(reqAnalysisService.제품_버전별_요구사항별_담당자_목록(지라이슈_제품_및_제품버전_집계_요청));
    }

    // Analysis Cost
    @GetMapping("/jira/dashboard/req-updated-list")
    public ResponseEntity<Map<String,List<요구사항_지라이슈키별_업데이트_목록_데이터>>> 요구사항_지라이슈키별_업데이트_목록(@RequestParam List<String> issueList) {

        log.info("요구사항 분석 컨트롤러 :: 요구사항_지라이슈키별_업데이트_목록");

        return ResponseEntity.ok(reqAnalysisService.요구사항_지라이슈키별_업데이트_목록(issueList));
    }

    // Analysis Time, Analysis Scope

    @PostMapping("/jira/dashboard/reqStatus/aggsOfLinkedIssueAndSubtasks")
    public ResponseEntity<?> 현황관리_연결이슈_하위이슈_집계조회(
            @RequestBody ALM_이슈_제품_및_버전_집계_요청 종합_집계_요청, @RequestParam("cReqLink") Long 요구사항_링크) {

        log.info("요구사항 분석 컨트롤러 :: 현황관리_연결이슈_하위이슈_집계조회");

        버킷_집계_결과_목록_합계 현황관리_연결이슈_하위이슈_집계 = reqAnalysisService.현황관리_연결이슈_하위이슈_집계(종합_집계_요청, 요구사항_링크);

        return ResponseEntity.ok(현황관리_연결이슈_하위이슈_집계);
    }

    @GetMapping("/updated-alm-issues")
    public ResponseEntity<List<AlmIssueEntity>> findUpdatedIssues(
            @RequestParam("pdServiceId") Long pdServiceId,
            @RequestParam("pdServiceVersions") List<Long> pdServiceVersionLinks,
            @RequestParam Long yearsAgo) {

        log.info("요구사항 분석 컨트롤러 :: ALM_이슈_누적_업데이트_목록_조회");

        return ResponseEntity.ok(reqAnalysisService.findUpdatedIssues(pdServiceId, pdServiceVersionLinks, yearsAgo));
    }

    // Analysis Time
    @PostMapping("/alm-issues/aggregation/by-date")
    public ResponseEntity<버킷_집계_결과_목록_합계> ALM_이슈_일별_제품_버전_집계_조회(
            @RequestBody ALM_이슈_일별_제품_및_버전_집계_요청 ALM_이슈_일별_제품_및_버전_집계_요청) {
        log.info("요구사항 분석 컨트롤러 :: ALM_이슈_기준일별_제품_버전_집계_조회");

        return ResponseEntity.ok(reqAnalysisService.ALM_이슈_일별_제품_버전_집계_조회(ALM_이슈_일별_제품_및_버전_집계_요청));
    }

    @PostMapping("/alm-issues/updated-list/by-date")
    public ResponseEntity<List<AlmIssueEntity>> 업데이트된_ALM_이슈_기준일별_목록_조회(
            @RequestBody ALM_이슈_일별_제품_및_버전_집계_요청 ALM_이슈_일별_제품_및_버전_집계_요청) {

        log.info("요구사항 분석 컨트롤러 :: 업데이트된_ALM_이슈_기준일별_목록_조회");

        return ResponseEntity.ok(reqAnalysisService.업데이트된_ALM_이슈_기준일별_목록_조회(ALM_이슈_일별_제품_및_버전_집계_요청));
    }


    // Analysis Time
    @PostMapping("/jira/dashboard/standard-daily/updated-jira-issue")
    public ResponseEntity<List<AlmIssueEntity>>
    기준일자별_제품_및_제품버전목록_업데이트된_이슈조회(
            @RequestBody ALM_이슈_일별_제품_및_버전_집계_요청 ALM_이슈_일별_제품_및_버전_집계_요청){

        log.info("요구사항 분석 컨트롤러 :: 기준일자별_제품_및_제품버전목록_업데이트된_이슈조회");

        return ResponseEntity.ok(reqAnalysisService.지라이슈_기준일자별_제품_및_제품버전_업데이트된_이슈조회(ALM_이슈_일별_제품_및_버전_집계_요청));
    }

    // Analysis Time
    @PostMapping("/jira/dashboard/standard-daily/updated-ridgeline")
    public Map<Long, Map<String, Map<String,List<요구사항_별_업데이트_데이터>>>>  기준일자별_제품_및_제품버전목록_업데이트된_누적_이슈조회(
            @RequestBody ALM_이슈_일별_제품_및_버전_집계_요청 ALM_이슈_일별_제품_및_버전_집계_요청) {

        log.info("요구사항 분석 컨트롤러 :: 기준일자별_제품_및_제품버전목록_업데이트된_누적_이슈조회");

        return reqAnalysisService.요구사항별_업데이트_능선_데이터(ALM_이슈_일별_제품_및_버전_집계_요청);
    }

}
