package com.arms.api.issue.status.strategy;

import com.arms.api.issue.status.dto.IssueStatusDTO;
import com.arms.api.issue.status.dto.CloudJiraByIssueTypeIssueStatusDTO;
import com.arms.api.issue.status.vo.CloudJiraByIssueTypeIssueStatusVO;
import com.arms.api.issue.status.vo.IssueStatusVO;
import com.arms.api.serverinfo.model.서버정보_데이터;
import com.arms.api.util.alm.지라API_정보;
import com.arms.api.util.alm.지라유틸;
import com.arms.api.util.errors.ErrorLogUtil;

import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.core.ParameterizedTypeReference;
import org.springframework.stereotype.Component;
import org.springframework.web.reactive.function.client.WebClient;

import java.util.*;
import java.util.stream.Collectors;

@Slf4j
@Component
@AllArgsConstructor
public class CloudJiraIssueStatusStrategy implements IssueStatusStrategy {

    private final 지라유틸 지라유틸;
    private final 지라API_정보 지라API_정보;

    @Override
    public List<IssueStatusVO> getIssueStatusList(서버정보_데이터 서버정보) {
        return null;
    }

    @Override
    public List<CloudJiraByIssueTypeIssueStatusVO> getIssueStatusListByProject(서버정보_데이터 서버정보,
                                                                               String projectId) {

        String endpoint = 지라API_정보.getEndpoint().getProject() + "/" + projectId + "/statuses";
        WebClient webClient = 지라유틸.클라우드_통신기_생성(서버정보.getUri(), 서버정보.getUserId(), 서버정보.getPasswordOrToken());

        try {
            List<CloudJiraByIssueTypeIssueStatusDTO> issueStatusList
                    = 지라유틸.get(webClient, endpoint,
                            new ParameterizedTypeReference<List<CloudJiraByIssueTypeIssueStatusDTO>>() {}
                        ).block();

            return Optional.ofNullable(issueStatusList)
                    .orElse(Collections.emptyList())
                    .stream()
                    .filter(Objects::nonNull)
                    .map(this::convertCloudJiraByIssueTypeIssueStatusVO)
                    .collect(Collectors.toList());
        }
        catch (Exception e) {
            String errorMessage = ErrorLogUtil.exceptionLoggingAndReturn(e, this.getClass().getName(),
                    String.format("%s[%s] :: getIssueStatusListByProject Error :: projectId[%s]",
                                서버정보.getType(), 서버정보.getUri(), projectId));

            throw new IllegalArgumentException(errorMessage);
        }
    }

    private CloudJiraByIssueTypeIssueStatusVO convertCloudJiraByIssueTypeIssueStatusVO(CloudJiraByIssueTypeIssueStatusDTO cloudJiraByIssueTypeIssueStatusDTO) {
        return CloudJiraByIssueTypeIssueStatusVO.builder()
                .self(cloudJiraByIssueTypeIssueStatusDTO.getSelf())
                .id(cloudJiraByIssueTypeIssueStatusDTO.getId())
                .name(cloudJiraByIssueTypeIssueStatusDTO.getName())
                .subtask(cloudJiraByIssueTypeIssueStatusDTO.isSubtask())
                .statuses(this.convertIssueStatusVOList(cloudJiraByIssueTypeIssueStatusDTO.getStatuses()))
                .build();
    }

    private List<IssueStatusVO> convertIssueStatusVOList(List<IssueStatusDTO> issueStatusDTOList) {
        return Optional.ofNullable(issueStatusDTOList)
                .orElse(Collections.emptyList())
                .stream()
                .filter(Objects::nonNull)
                .map(this::converIssueStatusVO)
                .collect(Collectors.toList());
    }

    private IssueStatusVO converIssueStatusVO(IssueStatusDTO issueStatusDTO) {
        return IssueStatusVO.builder()
                .self(issueStatusDTO.getSelf())
                .id(issueStatusDTO.getId())
                .name(issueStatusDTO.getName())
                .description(issueStatusDTO.getDescription())
                .issueTypeId(issueStatusDTO.getIssueTypeId())
                .build();
    }
}
