package com.arms.api.issue.status.strategy;

import com.arms.api.issue.status.vo.CloudJiraByIssueTypeIssueStatusVO;
import com.arms.api.issue.status.vo.IssueStatusVO;
import com.arms.api.serverinfo.model.서버정보_데이터;
import com.arms.api.util.alm.레드마인API_정보;
import com.arms.api.util.alm.레드마인유틸;
import com.arms.api.util.errors.ErrorLogUtil;
import com.taskadapter.redmineapi.RedmineException;
import com.taskadapter.redmineapi.RedmineManager;
import com.taskadapter.redmineapi.bean.IssueStatus;
import lombok.AllArgsConstructor;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Objects;

import static java.util.stream.Collectors.toList;

@Component
@AllArgsConstructor
public class OnpremiseRedmineIssueStatusStrategy implements IssueStatusStrategy {

    private final 레드마인유틸 레드마인유틸;
    private final 레드마인API_정보 레드마인API_정보;

    @Override
    public List<IssueStatusVO> getIssueStatusList(서버정보_데이터 서버정보) {

        RedmineManager redmineManager = 레드마인유틸.레드마인_온프레미스_통신기_생성(서버정보.getUri(), 서버정보.getPasswordOrToken());

        try {
            List<IssueStatus> redmineIssueStatusList = redmineManager.getIssueManager().getStatuses();

            List<IssueStatusVO> issueStatusList = redmineIssueStatusList.stream()
                    .map(issueStatus -> this.convertIssueStatusVO(issueStatus, 서버정보.getUri()))
                    .filter(Objects::nonNull)
                    .collect(toList());

            return issueStatusList;
        }
        catch (RedmineException e) {
            String errorMessage =  ErrorLogUtil.exceptionLoggingAndReturn(e, this.getClass().getName(),
                    String.format("%s[%s] :: getIssueStatusList Error", 서버정보.getType(), 서버정보.getUri()));
            throw new IllegalArgumentException(errorMessage);
        }

    }

    @Override
    public List<CloudJiraByIssueTypeIssueStatusVO> getIssueStatusListByProject(서버정보_데이터 서버정보,
                                                                               String projectId) {
        return null;
    }

    private IssueStatusVO convertIssueStatusVO(IssueStatus issueStatus, String serverUri) {
        String self = 레드마인유틸.서버정보경로_체크(serverUri) +
                레드마인API_정보.아이디_대체하기(
                        레드마인API_정보.getEndpoint().getIssuestatus(), String.valueOf(issueStatus.getId())
                );

        return IssueStatusVO.builder()
                .self(self)
                .id(issueStatus.getId().toString())
                .name(issueStatus.getName()).build();
    }
}
