package com.arms.api.project.controller;

import com.arms.api.project.vo.ProjectVO;
import com.arms.api.project.service.ProjectService;
import com.arms.egovframework.javaservice.esframework.erros.response.ErrorCode;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.web.bind.annotation.*;

import java.util.List;


@Slf4j
@RestController
@RequestMapping("/{connectId}/jira/project")
@AllArgsConstructor
public class ProjectController {

    private final ProjectService projectService;

    @ResponseBody
    @RequestMapping(
            value = {"/{projectKeyOrId}"},
            method = {RequestMethod.GET}
    )
    public ProjectVO getProject(@PathVariable("connectId") Long connectId,
                                @PathVariable("projectKeyOrId") String projectKeyOrId) {

        if (connectId == null) {
            log.error("getProject Error: connectId " + ErrorCode.파라미터_서버_아이디_없음.getErrorMsg());
            throw new IllegalArgumentException("getProject Error: connectId " + ErrorCode.파라미터_서버_아이디_없음.getErrorMsg());
        }

        if (StringUtils.isEmpty(projectKeyOrId)) {
            log.error("getProject Error projectKeyOrId " + ErrorCode.파라미터_NULL_오류.getErrorMsg());
            throw new IllegalArgumentException("getProject Error projectKeyOrId " + ErrorCode.파라미터_NULL_오류.getErrorMsg());
        }

        return projectService.getProject(connectId, projectKeyOrId);
    }

    @ResponseBody
    @RequestMapping(
            value = {"/list"},
            method = {RequestMethod.GET}
    )
    public List<ProjectVO> getProjectList(@PathVariable("connectId") Long connectId) {

        if (connectId == null) {
            log.error("getProjectList Error: connectId " + ErrorCode.파라미터_서버_아이디_없음.getErrorMsg());
            throw new IllegalArgumentException("getProjectList Error: connectId " + ErrorCode.파라미터_서버_아이디_없음.getErrorMsg());
        }

        return projectService.getProjectList(connectId);
    }
}