package com.arms.api.project.service;

import com.arms.api.project.vo.ProjectVO;
import com.arms.api.project.strategy.*;
import com.arms.api.serverinfo.model.enums.서버유형_정보;
import com.arms.api.serverinfo.model.서버정보_데이터;
import com.arms.api.serverinfo.service.서버정보_서비스;
import com.arms.egovframework.javaservice.esframework.erros.response.ErrorCode;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.List;

@Slf4j
@Service
@AllArgsConstructor
public class ProjectService {

    private final 서버정보_서비스 서버정보_서비스;
    private final CloudJiraProjectStrategy cloudJiraProjectStrategy;
    private final OnpremiseJiraProjectStrategy onpremiseJiraProjectStrategy;
    private final OnpremiseRedmineProjectStrategy onpremiseRedmineProjectStrategy;

    private ProjectStrategyImpl settingProjectStrategy(서버정보_데이터 서버정보) {

        if (서버정보 == null || 서버정보.getType().isEmpty()) {
            log.error("프로젝트 전략 등록 Error: 서버정보_유형 " + ErrorCode.서버유형_정보오류.getErrorMsg());
            throw new IllegalArgumentException("프로젝트 전략 등록 Error: 서버정보_유형 " + ErrorCode.서버유형_정보오류.getErrorMsg());
        }

        서버유형_정보 서버_유형 = 서버유형_정보.valueOf(서버정보.getType());

        ProjectStrategy projectStrategy;
        if (서버_유형 == 서버유형_정보.클라우드) {
            projectStrategy = this.cloudJiraProjectStrategy;
        }
        else if (서버_유형 == 서버유형_정보.온프레미스) {
            projectStrategy = this.onpremiseJiraProjectStrategy;
        }
        else if (서버_유형 == 서버유형_정보.레드마인_온프레미스) {
            projectStrategy = this.onpremiseRedmineProjectStrategy;
        }
        else {
            throw new IllegalArgumentException("프로젝트 전략 확인 Error: 허용하지 않는 서버정보_유형입니다. :: "+ 서버_유형+ " :: " + ErrorCode.서버유형_정보오류.getErrorMsg());
        }

        ProjectStrategyImpl projectStrategyImpl = new ProjectStrategyImpl();
        projectStrategyImpl.registerProjectStrategy(projectStrategy);

        return projectStrategyImpl;
    }

    public ProjectVO getProject(Long connectId, String projectKeyOrId) {

        서버정보_데이터 서버정보 = 서버정보_서비스.서버정보_검증(connectId);

        ProjectStrategyImpl projectStrategy = this.settingProjectStrategy(서버정보);;
        return projectStrategy.getProject(서버정보, projectKeyOrId);
    }

    public List<ProjectVO> getProjectList(Long connectId) {

        서버정보_데이터 서버정보 = 서버정보_서비스.서버정보_검증(connectId);

        ProjectStrategyImpl projectStrategy = this.settingProjectStrategy(서버정보);;
        return projectStrategy.getProjectList(서버정보);
    }
}